/* findutils-version.h -- show version information for findutils
   Copyright (C) 2007-2022 Free Software Foundation, Inc.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/
#ifndef FINDUTILS_VERSION_H
# define FINDUTILS_VERSION_H
/*
 * display_findutils_version displays GNU coding standard compliant
 * version information.
 *
 * official_name   Official name of the program, for example
 *                 "find" or "xargs" (as opposed to "/bin/find"
 *                 or gfind).
 *
 */
void display_findutils_version(const char *official_name);

#endif
